import boto3
import os
import json
from botocore.exceptions import ClientError
AWS_SECRET_ACCESS_KEY=os.environ.get('aws_secret_key')
AWS_ACCESS_KEY_ID=os.environ.get('aws_access_key')




class AWS:
    def __init__(self, **args):
        self.service_name = args.get('service_name')
        self.operation = args.get('operation')
        self.region_name = args.get('region')
        self.server_name = args.get('name')

    def invoke_service(self):   
        try:
            if self.service_name and self.region_name is not None:
                print('Request for External AWS API.......')
                service_function = {
                    "ec2": 'manage_ec2',
                    "amazonec2": 'manage_ec2',
                    "s3": 'manage_s3',
                    "amazon ec2": 'manage_ec2'
                }
                service_found = False
                for service, function_name in service_function.items():
                     if any(keyword.lower() in self.service_name.lower() for keyword in service.split()):
                            service_found = True
                            self.service_name = service
                            session = boto3.resource(
                                self.service_name,
                                aws_access_key_id=AWS_ACCESS_KEY_ID,
                                aws_secret_access_key=AWS_SECRET_ACCESS_KEY,
                                region_name=self.region_name
                        )
                            service_exec=getattr(self, service_function[self.service_name])(self.server_name, session, self.region_name, self.operation)
                            return service_exec

            else:
                print('Service variables are not set. Please check the service name and region name.')
                return
    
        except Exception as e:
            print('Error running AWS api request')
            print(e)

    def list_recent_ami(self, region_name):
        ec2 = boto3.client('ec2', region_name=region_name, aws_access_key_id=AWS_ACCESS_KEY_ID,
                                aws_secret_access_key=AWS_SECRET_ACCESS_KEY)
        response = ec2.describe_images(
            Filters=[
                {
                    'Name': 'architecture',
                    'Values': ['x86_64']  # Optionally, filter by architecture
                },
                {
                    'Name': 'state',
                    'Values': ['available']  # Optionally, filter by state
                },
                {
                    'Name': 'owner-alias',
                    'Values': ['amazon']  # Optionally, filter by owner alias
                },
                {
                    'Name': 'root-device-type',
                    'Values': ['ebs']  # Optionally, filter by root device type
                },
                {
                    'Name': 'virtualization-type',
                    'Values': ['hvm']  # Filter by HVM virtualization type
                },
                {
                    'Name': 'name',
                    'Values': ['al2023-ami-2023.3.*']  # Filter by name containing "al2023"
                }
            ]
        )
        # Sort images by creation date
        sorted_images = sorted(response['Images'], key=lambda x: x['CreationDate'], reverse=True)

        # Get the most recent image
        most_recent_image = sorted_images[0]['ImageId'] if sorted_images else None

        return most_recent_image

    def manage_ec2(self, server_name, ec2, region_name, operation=None):
        api_result = {}
        #action = "create_instances" if any(word.lower() in operation.lower() for word in ["build","create","deploy","make","construct","generate","produce","design"]) else "terminate_instances"
        try:
            ami_id = self.list_recent_ami(region_name)
            api_result = ec2.create_instances(
                ImageId=ami_id,
                InstanceType="t2.micro",
                KeyName="apikey", 
                MinCount=1,
                MaxCount=1
            )[0]
            api_result.wait_until_running()
            api_result.create_tags(
                    Tags=[
                        {
                            'Key': 'Name',
                            'Value': server_name
                            },
            # Add any additional tags here if needed
            ]
        )
            api_result.load()
            # print(api_result)
            instance = ec2.Instance(api_result.id)
            instance_details = {
                "id": instance.id,
                "Type": instance.instance_type,
                "State": instance.state['Name'],
                "Public IP": instance.public_ip_address,
            }
            return instance_details

            # return api_result
        except ClientError as e:
            print(e)

    def manage_s3(self, server_name, s3, region_name, operation=None):
        api_result = {}
        api_result = s3.create_bucket(Bucket=server_name,
                                    ACL='private',
                                    CreateBucketConfiguration={
                                        'LocationConstraint': region_name
                                    })
        api_result.wait_until_exists()
        api_result.load()

        print(api_result) 


from .features import Service


class shopping(Service):

    @property
    def service(self):
        return "GoogleShopping"
    
    @property
    def content(self):
        return "You are a shopping assistant. Provide me the top 3 shopping results. Only get the title, price and link for each item in natural language."
    
    @property
    def tools(self):
        return   {
      "name": "extractShoppingDetail",
      "description": "Extract shopping detail from the user prompt",
      "parameters": {
        "type": "object",
        "properties": {
          "item": {
            "type": "string",
            "description": "The item name"
          },
          "amount": {
            "type": "number",
            "description": "The amount of the item"
          },
          "brand": {
            "type": "string",
            "description": "The brand of the item"
          },
          "price": {
            "type": "number",
            "description": "The price of the item"
          }
        },
        "required": ["query"]
      }
    }


from abc import ABC

class Service(ABC):
    
    @property
    def service(self):
        return self.service
    
    @property
    def content(self):
        return self.content
    
    @property
    def tools(self):
        return self.tools


import os
import json
import serpapi

serpapi_key = os.environ.get('serpapi_key')

class GoogleShopping:
    def __init__(self, **args):
        self.item = args.get('item')
        self.brand = args.get('brand')
        self.price = args.get('price')
        self.amount = args.get('amount')

    def invoke_service(self):
        try:
            search = serpapi.GoogleSearch({
                "engine": "google_shopping",
                "q": self.brand + " " + self.item,
                "hl": "en",
                "gl": "in",
                "tbm": "mr:1,price:1,ppr_max:{}".format(self.price),
                "api_key": os.environ.get('serpapi_key')
            })

            results = search.get_dict()
            products = results['shopping_results'][:3]
            # print(products)
            return products
        
        except Exception as e:
            print('Error extracting shopping details')
            print(e)
            return


from setuptools import setup, find_packages

VERSION = '0.0.7' 
DESCRIPTION = 'GPT Assistant'
LONG_DESCRIPTION = 'An AI assistant to help with several services.'

# Setting up
setup(
       # the name must match the folder name 'verysimplemodule'
        name="gptservice", 
        version=VERSION,
        author="Sukamal Rakshit",
        author_email="sukamalrksht77@email.com",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        entry_points={
        'console_scripts': [
            'gptservice=gptservice.apps:main',
        ],
    },
        install_requires=['boto3',
                          'serpapi',
                          'openai'],
        
        keywords=['python', 'chatbot'],
        classifiers= [
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Education",
            "Programming Language :: Python :: 3",
            "Operating System :: POSIX :: Linux",
            "Operating System :: Microsoft :: Windows",
        ]
)


from .features import Service


class aws(Service):

    @property
    def service(self):
        return "AWS"
    
    @property
    def content(self):
        return "As a Cloud service assistant, you must have the service_name and region to construct any service. These two details are mandatory, and if any of those two details are missing, prompt for both service and region. Additionally, if the output contains string 'Public IP', it means that the resource has been constructed. In such cases, return the output with details like public IP, id, and the state of the resource in a natural language always."
    
    @property
    def tools(self):
        return  {
            "name": "invoke_aws_service",
            "description": "Invoke an AWS service",
            "parameters": {
                "type": "object",
                "properties": {
                    "service_name": {
                        "type": "string",
                        "description": "The name of the AWS service to invoke"
                    },
                    "operation": {
                        "type": "string",
                        "description": "Build or Terminate a service"
                    },
                    "region": {
                        "type": "string",
                        "description": "The AWS region to use for the service call"
                    },

                    "name": {
                        "type": "string",
                        "description": "The name of the instance"
                    }
                }
            },
            "required": ["service_name", "operation", "region"]
            
        }


import json
import os
import boto3
import sys
from openai import OpenAI
from .Tools.services import SERVICES
from .Tools.features.aws_services import AWS
from .Tools.features.google_shopping import GoogleShopping

AWS_SECRET_ACCESS_KEY=os.environ.get('aws_secret_key')
AWS_ACCESS_KEY_ID=os.environ.get('aws_access_key')
API_KEY=os.environ.get('openai_key')
GPT_MODEL = "gpt-3.5-turbo-0613"
tools = [
    SERVICES['AWS'].tools,SERVICES['GoogleShopping'].tools
        
]  
client = OpenAI(api_key=API_KEY)


def call_assistant(tools, userPrompt):
    chat_response = client.chat.completions.create(
        model=GPT_MODEL,
        messages=[{"role": "system", "content": "Hello, I'm an AI assistant developed by OpenAI. Sukamal is the developer of this API service. I would like to discuss its functionality with you. The main features of the service include AWS Services like ec2 and s3 as of now and Shopping Assistant."},
                  {"role": "user", "content": userPrompt}
                  ],
        functions=tools,
        function_call='auto'
    )
    try:  
        assistant_message = json.loads(chat_response.choices[0].message.function_call.arguments)
        if not all((assistant_message)):
            assistant_message = chat_response.choices[0].message.content
            return assistant_message
        else:
             message = process_service(assistant_message, userPrompt)
             return message
            # item,amount,brand,price = extract_payload(chat_response)
            # extractShoppingDetail(item, brand, price, amount)
            # assistant_input = invoke_aws(service_name, region_name, server_name, operation)
            # return assistant_input

    except Exception as e:
        assistant_message = chat_response.choices[0].message.content
        return assistant_message

def process_service(assistant_message, userPrompt):
                SERVICES_CONFIG = {
                    'AWS': ['service_name', 'operation', 'region', 'name'],
                    'GoogleShopping': ['item', 'brand', 'price', 'amount']
        # Add more services and their parameters as needed
                }
                # Extract service and params from assistant message
                service, *required_params = extract_payload(assistant_message)
                if not all(required_params):
                    content = SERVICES.get(service).content
                    assistant_message = natural_language_processing(userPrompt, content)
                    return assistant_message
                
                elif all(required_params):
                    payload_params = SERVICES_CONFIG[service]
                    payload = {param: value for param, value in zip(payload_params, required_params)}
                    service_class = getattr(sys.modules[__name__], service, None)

                    # Check if the service class exists
                    if service_class is not None:
                    # If it exists, instantiate it with the payload
                        trigger = service_class(**payload)
                        results = trigger.invoke_service()
                        content = SERVICES.get(service).content
                        assistant_response = natural_language_processing(results, content)
                        return assistant_response
                    else:
                    # If it doesn't exist, raise an error or handle it appropriately
                        raise ValueError(f'Invalid service name: {service}')

                    

def extract_payload(assistant_message):
    try:
        # assistant_message = json.loads(chat_response.choices[0].message.function_call.arguments)
        # print(assistant_message)
        if 'service_name' in assistant_message:
            service = 'AWS'
            service_name = assistant_message.get('service_name').lower()
            operation = assistant_message.get('operation', 'build')
            region = assistant_message.get('region', '')
            name = assistant_message.get('name', 'gpt-test')
            return service, service_name, operation, region, name
                
        elif 'item' in assistant_message:
            service = 'GoogleShopping'
            item = assistant_message.get('item','')
            amount = assistant_message.get('amount', 1)
            brand = assistant_message.get('brand','')
            price = assistant_message.get('price','')
            return service, item, brand, amount, price
        
        else:
            return
    except Exception as e:
        print(e)
        return


def natural_language_processing(results,content):
    response = client.chat.completions.create(
        model=GPT_MODEL,
        messages=[{"role": "system", "content": content},
                  {"role": "user", "content": json.dumps(results)}]
    )
    return response.choices[0].message.content

def main(): 
       
    while True:
        userPrompt = input("Enter your prompt ('q' to quit): ")
        if userPrompt.lower() == 'q':
            break
        api_out = call_assistant(tools, userPrompt)
        print(api_out)
        # assistant_response = natural_language_processing(api_out)
        # print(assistant_response)
if __name__ == "__main__":
     main()
    





